//  *******************************************************
//                gui.c
//  *******************************************************

#include "gui.h"
#include "lcd.h"

/**
* This Function fills a spezified bar defined by the index of the bar (0-BARCOUNT)
* 
* @param nBarNumber Index of the bar that has to be filled
* @param nPercent Fill percentage of the bar in (0-100)
*/
void drawBar(unsigned char nBarNumber, unsigned char nPercent)
{
	int x1,x2,y1,y2;

	if(((nBarNumber+1) <= BARCOUNT) && (nPercent <= BARHEIGHT))
	{

		//calculate point p1(x1/x2)
		x1 = XSPACE + (BARHEIGHT - nPercent);
		y1 = YSPACE + (nBarNumber * (BARWIDTH + BARSPACE));

		//calculate point p2(x1/x2)
		x2 = x1 + nPercent;
		y2 = y1 + BARWIDTH;

		//clear the draw area
		LCDSetRect(XSPACE,y1,XSPACE+BARHEIGHT,y2,1,BACKCOLOR);
		//draw Bar
		LCDSetRect(x1,y1,x2,y2,1,BARCOLOR);
	}
}

/**
* This function sets the name of the onscreen buttons
* 
* @param sButtonText the button text
* @param nSize the font size given by the display driver (SMALL, MEDIUM,LARGE)
* @param nButton the value should be LEFTB or RIGHTB
*/
void drawButtonLabel(unsigned char* sButtonText, int nSize, unsigned char nButton)
{
	switch(nButton) 
	{
	case LEFTB: LCDPutStr(sButtonText, BUTTONXPOS, LEFTB, nSize, MAINCOLOR, BACKCOLOR); break;
	case RIGHTB: LCDPutStr(sButtonText, BUTTONXPOS, RIGHTB, nSize, MAINCOLOR, BACKCOLOR); break;
		//default: ;
	}
}

/**
* This function displays an array of bars
* 
* @param pBars pointer to an arrys data structure (has to be BARCOUNT elements big)
*/
void showBars(unsigned char* pBars)
{
	unsigned char i;
	for(i=0;i<BARCOUNT;i++)
	{
		drawBar(i,pBars[i]);
	}
}

/**
* This function draws the basic GUI
*
* The Headline, Bars and Buttons
*
* (for future version the Headline should be rewrite able)
*/
void drawGUI(void)
{
	unsigned char pInitBars[BARCOUNT];
	int i=0;
	for(i=0;i<BARCOUNT;i++)
	{
		pInitBars[i] = 0;
	}
	//set background color
	LCDSetRect(0,0,131,131, 1,BACKCOLOR);

	//set copyright
	LCDPutStr("Equalizer (c)2008 FAG", 4, 1, SMALL, MAINCOLOR, BACKCOLOR);

	//set all bars to zero
	showBars(pInitBars);
	//draw a line under the bars
	LCDSetLine(LINEPOS, 0, LINEPOS, 131, MAINCOLOR);

	//draw horizontal line between button text
	LCDSetLine(LINEPOS, RIGHTB-2, 131, RIGHTB-2, MAINCOLOR);

	//draw button text
	drawButtonLabel("START", LARGE, LEFTB);
	drawButtonLabel("STOP", LARGE, RIGHTB);
}
