//  *****************************************************************************
//   						main.c
// 
//		Author:  Christian Grnke - Feb 07
//  *****************************************************************************

//  *******************************************************
//                Header Files
//  *******************************************************
#include "AT91SAM7X256.h"
#include "lcd.h"
#include "board.h"

// sprintf etc.
//#include <stdio.h>
#include <stdlib.h>

#include "functions.h"
#include "gui.h"


//  *******************************************************
//                External References
//  *******************************************************
extern	void LowLevelInit(void);

//  *******************************************************
//                Test func
//  *******************************************************

void delaysec(void) {
	volatile AT91PS_TC pTC2 = AT91C_BASE_TC2;

	pTC2->TC_CCR = 0x5;
	//1/4 sec
	while(pTC2->TC_CV <= 11700){}
}

//  *******************************************************
//                Interrupt
//  *******************************************************

// *** FLAGS ***
unsigned char timer2_reached = 0;
// *** INTERRUPT HANDELS ***

__attribute__ ((interrupt ("IRQ"))) void handleTC2_IRQ(void) {
	unsigned int dummy;

	// *** POINTERS ***
	volatile AT91PS_AIC pAIC = AT91C_BASE_AIC;
	//volatile AT91PS_TC pTimer = AT91C_BASE_TC0 + ((1 >> (pAIC->AIC_IPR))*0x40);
	//test
	volatile AT91PS_TC pTimer = AT91C_BASE_TC2;
	
	if(timer2_reached == 1) {
		timer2_reached = 0;
	} else {
		timer2_reached = 1;
	}

	dummy = pTimer->TC_SR;
	dummy = dummy;

	//end Interrupt Handling
	//write something to the AIC_EOICR register
	//pAIC->AIC_EOICR = 0;
}

//  *******************************************************
//                Timer
//  *******************************************************
#define TIMER_CLOCK1 0x0
#define TIMER_CLOCK2 0x1
#define TIMER_CLOCK3 0x2
#define TIMER_CLOCK4 0x3
#define TIMER_CLOCK5 0x4

void initTimer(unsigned short nValue, volatile AT91PS_TC pTimer, unsigned char nTimerID, unsigned char nClockSel, unsigned char nRepeat, void (*pHandle) (void)) {
	if(nRepeat <= 1) {
		volatile AT91PS_AIC pAIC = AT91C_BASE_AIC;
		volatile AT91PS_PMC pPMC = AT91C_BASE_PMC;

		//Enable Timer Clock
		pPMC->PMC_PCER = pPMC->PMC_PCSR | (0x1 << nTimerID);

		//Init Timer
		pTimer->TC_CMR = ((nRepeat << 14) | (nClockSel));

		//Init Counter Interrupt
		pAIC->AIC_IDCR = (0x1 << nTimerID);
		pAIC->AIC_SMR[nTimerID] = (AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE | 0x7);
		pAIC->AIC_SVR[nTimerID] = (unsigned int)pHandle;
		pAIC->AIC_ICCR = (0x1 << nTimerID);

		//Enable Interrupt
		pTimer->TC_IER = 0x10;
		pAIC->AIC_IECR = (0x1 << nTimerID);

		//Set Register C (Counter Stop)
		pTimer->TC_RC = nValue;

		//Start Timer
		pTimer->TC_CCR = 0x5;
	}
}

int	main (void) {

	/* Variablen */
	int IdleCount = 0;
	char teststring[250];
	int i=0;

	unsigned char a[] = {5,10,27,68,6,10,7,99,16,33};
	unsigned char b[] = {22,1,0,45,70,84,3,30,57,2};
	unsigned char c[] = {3,28,57,37,60,43,16,97,32,37};

	// *** POINTERS ***
	volatile AT91PS_AIC pAIC = AT91C_BASE_AIC;
	volatile AT91PS_PMC pPMC = AT91C_BASE_PMC; //Pointer to PMC (Powermanagement)
	volatile AT91PS_TC pTC2 = AT91C_BASE_TC2;
	volatile AT91PS_PIO	pPB = AT91C_BASE_PIOB; //pointer to PIO B
	//volatile AT91PS_ADC pADC = AT91C_BASE_ADC;

	// Initialize the Atmel AT91SAM7S256 (watchdog, PLL clock, default interrupts, etc.)
	LowLevelInit();

	// Initialize SPI interface to LCD
	InitSpi();

	// Init LCD
	InitLcd();

	// clear the screen
	LCDClearScreen();

	/* *** INIT TEST *** */

	//Activate Power for PIO
	pPMC->PMC_PCER = 0xC; //enable PIO B Clock --- PIO B ID 3 --- PIO A ID 2

	//Activate Power for TC
	pPMC->PMC_PCER = pPMC->PMC_PCSR | 0x4000;

	//Init Timer
	pTC2->TC_CMR = 0x4;
	pTC2->TC_IDR = 0xF;

	//Set Buttons
	//pPB->PIO_PER = pPB->PIO_PSR | SW1_MASK | SW2_MASK; //Enable PIO Register for Port
	//pPB->PIO_ODR = ~(pPB->PIO_OSR) | SW1_MASK | SW2_MASK; //Disable Output
	//pPB->PIO_PPUDR = ~(pPB->PIO_PPUSR) | SW1_MASK | SW2_MASK; //Disable Pullup Pullup
	//pPB->PIO_IFER = pPB->PIO_IFSR | SW1_MASK | SW2_MASK; //Enable Glitch Filter
	//pPB->PIO_IDR = ~(pPB->PIO_IMR) | SW2_MASK; //Interrupt Disable
	//pPB->PIO_MDDR = ~(pPB->PIO_MDSR) | SW1_MASK | SW2_MASK; //Multidriver disable
	//pPB->PIO_OWDR = ~(pPB->PIO_OWSR) | SW1_MASK | SW2_MASK; //Outputwrite disable

	//Interrupt Test
	//pAIC->AIC_IDCR = ((0x1 << AT91C_ID_PIOB) | SW1_MASK); //disable IRQ
	//pAIC->AIC_SMR[3] = AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE; //Interrupt PRIOR=0 positive edge triggered
	//pAIC->AIC_SVR[3] = (unsigned int)handlePIO_IRQ;
	//pAIC->AIC_SMR[SW1] = AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE; //Interrupt PRIOR=0 positive edge triggered
	//pAIC->AIC_SVR[SW1] = (unsigned int)handlePIO_IRQ;
	//pAIC->AIC_ICCR = ((0x1 << AT91C_ID_PIOB) | SW1_MASK); //clear IRQ

	//	
	////enable SW1 interrupt
	//pPB->PIO_IER = pPB->PIO_IMR | SW1_MASK;

	//pAIC->AIC_IECR = ((0x1 << AT91C_ID_PIOB)| SW1_MASK); //enable IRQ

	////Init AD7 - Test
	//pADC->ADC_CR = 0x2; //Start AD Wandlung
	//pADC->ADC_MR = 0x3F00; //Set Options (see Portmap.xls)
	//pADC->ADC_CHER = 0x80; //Enable Channel AD7
	//pADC->ADC_CHDR = 0x7F; //Disable Cannel AD0-AD6
	//pADC->ADC_IDR = 0xFFFF; //Disable Interrupt
	
	/* *** HERE STARTS SOME TESTS *** */

	//draw basic GUI
	drawGUI();

	initTimer(50000, AT91C_BASE_TC2, AT91C_ID_TC2,TIMER_CLOCK5, 1, handleTC2_IRQ);

	/*** MAINLOOP ***/
	while (1) {

		//pAIC->AIC_ISCR = 0xFFFFFFFC;

		ItoA(pTC2->TC_CV, teststring);
		LCDPutStr(teststring, 60, 20, SMALL, WHITE, BLACK);

		ItoA(timer2_reached, teststring);
		LCDPutStr(teststring, 40, 20, SMALL, WHITE, BLACK);
		
		/*
		ItoA(pPB->PIO_PDSR & SW1_MASK, teststring);
		LCDPutStr(teststring, 40, 10, SMALL, WHITE, BLACK);
		*/

		//pADC->ADC_CR = 0x2; //Start AD Wandlung
		//while(!((pADC->ADC_SR & 0x80) == 0x80)){}
		//
		//ItoA(pADC->ADC_CDR7, teststring);
		//LCDPutStr(teststring, 40, 10, SMALL, WHITE, BLACK);
		//
		//LCDClearScreen();

		IdleCount++;									// count # of times through the idle loop
	}
}
